﻿using System;
using System.Collections.Generic;
using System.Linq;
using Verse;
using Harmony;
using System.Reflection;
using RimWorld;

namespace RealisticStarvation
{
    [StaticConstructorOnStartup]
    class Main
    {
        static Main()
        {
            var harmony = HarmonyInstance.Create("com.gametheatre.mods.rimworld.harmonytest");
            harmony.PatchAll(Assembly.GetExecutingAssembly());
        }

        [HarmonyPatch(typeof(Need_Food))]
        [HarmonyPatch("NeedInterval")]
        static class NeedInterval_Patch
        {
            static float MalnutritionSeverityPerInterval = 0.00033333332f;

            static bool Prefix(Need_Food __instance)
            {
                if ((bool)GetPrivateProperty(typeof(Need), "IsFrozen", __instance))
                {
                    //Log.Message((GetPrivateField(typeof(Need_Food), "pawn", __instance) as Pawn).Name + " has frozen needs");
                }
                else
                {
                    __instance.CurLevel = __instance.CurLevel - __instance.FoodFallPerTick * 150f;
                }

                if (!__instance.Starving)
                {
                    SetPrivateField(typeof(Need_Food), "lastNonStarvingTick", Find.TickManager.TicksGame, __instance);
                    HealthUtility.AdjustSeverity((Pawn) GetPrivateField(typeof(Need_Food), "pawn", __instance),
                        HediffDefOf.Malnutrition,
                        -MalnutritionSeverityPerInterval * ReduceFactorPerTickAssumingCategory(__instance.CurCategory));
                }
                else
                {
                    HealthUtility.AdjustSeverity((Pawn) GetPrivateField(typeof(Need_Food), "pawn", __instance),
                        HediffDefOf.Malnutrition, MalnutritionSeverityPerInterval);
                }
                return false;
            }
        }

        [HarmonyPatch(typeof(Need_Food))]
        [HarmonyPatch("FoodFallPerTickAssumingCategory")]
        static class FoodFallPerTickAssumingCategory_Patch
        {
            static float BaseFoodFallPerTick = 2.66666666E-05f;

            static bool Prefix(Need_Food __instance, ref float __result, HungerCategory cat)
            {
                float hungerRate = (float) (GetPrivateProperty(typeof(Need_Food), "HungerRate", __instance));

                switch (cat)
                {
                    case HungerCategory.Fed:
                    {
                        __result = BaseFoodFallPerTick * hungerRate;
                        break;
                    }
                    case HungerCategory.Hungry:
                    {
                        __result = BaseFoodFallPerTick * hungerRate * 0.3f;
                        break;
                    }
                    case HungerCategory.UrgentlyHungry:
                    {
                        __result = BaseFoodFallPerTick * hungerRate * 0.1f;
                        break;
                    }
                    case HungerCategory.Starving:
                    {
                        __result = BaseFoodFallPerTick * hungerRate * 0.05f;
                        break;
                    }
                }

                return false;
            }
        }

        static float ReduceFactorPerTickAssumingCategory(HungerCategory cat)
        {
            switch (cat)
            {
                case HungerCategory.Fed:
                {
                    return 1.5f;
                }
                case HungerCategory.Hungry:
                {
                    return 0.5f;
                }
                case HungerCategory.UrgentlyHungry:
                {
                    return 0.25f;
                }
                case HungerCategory.Starving:
                {
                    return 0.1f;
                }
            }
            return 999f;
        }

        public static void SetPrivateField(Type t, string varName, object varValue, object objInstance)
        {
            BindingFlags eFlags = BindingFlags.Instance | BindingFlags.Public | BindingFlags.NonPublic;
            var fi = t.GetField(varName, eFlags);
            fi?.SetValue(objInstance, varValue);
        }

        public static object GetPrivateField(Type t, string varName, object objInstance)
        {
            BindingFlags eFlags = BindingFlags.Instance | BindingFlags.Public | BindingFlags.NonPublic;
            var fi = t.GetField(varName, eFlags);
            return fi?.GetValue(objInstance);
        }

        public static object GetPrivateProperty(Type t, string varName, object objInstance)
        {
            BindingFlags eFlags = BindingFlags.Instance | BindingFlags.Public | BindingFlags.NonPublic;
            var pi = t.GetProperty(varName, eFlags);
            return pi?.GetGetMethod(true)?.Invoke(objInstance, null);
        }
    }
}